/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSAppleScript;

import dstep.foundation.NSAppleEventDescriptor;
import dstep.foundation.NSDictionary;
import dstep.foundation.NSObject;
import dstep.foundation.NSString;
import dstep.foundation.NSURL;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc : id;

import bindings = dstep.foundation.NSAppleScript_bindings;

const NSString NSAppleScriptErrorMessage;
const NSString NSAppleScriptErrorNumber;
const NSString NSAppleScriptErrorAppName;
const NSString NSAppleScriptErrorBriefMessage;
const NSString NSAppleScriptErrorRange;

static this ()
{
	NSAppleScriptErrorMessage = new NSString(bindings.NSAppleScriptErrorMessage);
	NSAppleScriptErrorNumber = new NSString(bindings.NSAppleScriptErrorNumber);
	NSAppleScriptErrorAppName = new NSString(bindings.NSAppleScriptErrorAppName);
	NSAppleScriptErrorBriefMessage = new NSString(bindings.NSAppleScriptErrorBriefMessage);
	NSAppleScriptErrorRange = new NSString(bindings.NSAppleScriptErrorRange);
}

class NSAppleScript : NSObject, INSCopying
{
	mixin ObjcWrap;

	Object initWithContentsOfURL (NSURL url, NSDictionary** errorInfo)
	{
		return invokeObjcSelf!(Object, "initWithContentsOfURL:error:", NSURL, NSDictionary**)(url, errorInfo);
	}

	this (NSURL url, NSDictionary** errorInfo)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithContentsOfURL:error:", NSURL, NSDictionary**)(objcObject, url, errorInfo);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	Object initWithSource (NSString source)
	{
		return invokeObjcSelf!(Object, "initWithSource:", NSString)(source);
	}

	this (NSString source)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithSource:", NSString)(objcObject, source);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	NSString source ()
	{
		return invokeObjcSelf!(NSString, "source");
	}

	bool isCompiled ()
	{
		return invokeObjcSelf!(bool, "isCompiled");
	}

	bool compileAndReturnError (NSDictionary** errorInfo)
	{
		return invokeObjcSelf!(bool, "compileAndReturnError:", NSDictionary**)(errorInfo);
	}

	NSAppleEventDescriptor executeAndReturnError (NSDictionary** errorInfo)
	{
		return invokeObjcSelf!(NSAppleEventDescriptor, "executeAndReturnError:", NSDictionary**)(errorInfo);
	}

	NSAppleEventDescriptor executeAppleEvent (NSAppleEventDescriptor event, NSDictionary** errorInfo)
	{
		return invokeObjcSelf!(NSAppleEventDescriptor, "executeAppleEvent:error:", NSAppleEventDescriptor, NSDictionary**)(event, errorInfo);
	}

	Object copyWithZone (NSZone* zone)
	{
		return invokeObjcSelf!(Object, "copyWithZone:", NSZone*)(zone);
	}
}

