/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSArchiver;

import dstep.foundation.NSCoder;
import dstep.foundation.NSData;
import dstep.foundation.NSMutableArray;
import dstep.foundation.NSMutableData;
import dstep.foundation.NSMutableDictionary;
import dstep.foundation.NSString;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc : id;

import bindings = dstep.foundation.NSArchiver_bindings;

const NSString NSInconsistentArchiveException;

static this ()
{
	NSInconsistentArchiveException = new NSString(bindings.NSInconsistentArchiveException);
}

class NSArchiver : NSCoder
{
	mixin ObjcWrap;

	Object initForWritingWithMutableData (NSMutableData mdata)
	{
		return invokeObjcSelf!(Object, "initForWritingWithMutableData:", NSMutableData)(mdata);
	}

	this (NSMutableData mdata)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initForWritingWithMutableData:", NSMutableData)(objcObject, mdata);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	NSMutableData archiverData ()
	{
		return invokeObjcSelf!(NSMutableData, "archiverData");
	}

	void encodeRootObject (Object rootObject)
	{
		return invokeObjcSelf!(void, "encodeRootObject:", Object)(rootObject);
	}

	void encodeConditionalObject (Object object)
	{
		return invokeObjcSelf!(void, "encodeConditionalObject:", Object)(object);
	}

	static NSData archivedDataWithRootObject (Object rootObject)
	{
		return invokeObjcSelfClass!(NSData, "archivedDataWithRootObject:", Object)(rootObject);
	}

	static bool archiveRootObject (Object rootObject, NSString path)
	{
		return invokeObjcSelfClass!(bool, "archiveRootObject:toFile:", Object, NSString)(rootObject, path);
	}

	void encodeClassName (NSString trueName, NSString inArchiveName)
	{
		return invokeObjcSelf!(void, "encodeClassName:intoClassName:", NSString, NSString)(trueName, inArchiveName);
	}

	NSString classNameEncodedForTrueClassName (NSString trueName)
	{
		return invokeObjcSelf!(NSString, "classNameEncodedForTrueClassName:", NSString)(trueName);
	}

	void replaceObject (Object object, Object newObject)
	{
		return invokeObjcSelf!(void, "replaceObject:withObject:", Object, Object)(object, newObject);
	}
}

class NSUnarchiver : NSCoder
{
	mixin ObjcWrap;

	Object initForReadingWithData (NSData data)
	{
		return invokeObjcSelf!(Object, "initForReadingWithData:", NSData)(data);
	}

	this (NSData data)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initForReadingWithData:", NSData)(objcObject, data);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	void setObjectZone (NSZone* zone)
	{
		return invokeObjcSelf!(void, "setObjectZone:", NSZone*)(zone);
	}

	NSZone* objectZone ()
	{
		return invokeObjcSelf!(NSZone*, "objectZone");
	}

	bool isAtEnd ()
	{
		return invokeObjcSelf!(bool, "isAtEnd");
	}

	uint systemVersion ()
	{
		return invokeObjcSelf!(uint, "systemVersion");
	}

	static Object unarchiveObjectWithData (NSData data)
	{
		return invokeObjcSelfClass!(Object, "unarchiveObjectWithData:", NSData)(data);
	}

	static Object unarchiveObjectWithFile (NSString path)
	{
		return invokeObjcSelfClass!(Object, "unarchiveObjectWithFile:", NSString)(path);
	}

	static void decodeClassName (NSString inArchiveName, NSString trueName)
	{
		return invokeObjcSelfClass!(void, "decodeClassName:asClassName:", NSString, NSString)(inArchiveName, trueName);
	}

	void decodeClassName (NSString inArchiveName, NSString trueName)
	{
		return invokeObjcSelf!(void, "decodeClassName:asClassName:", NSString, NSString)(inArchiveName, trueName);
	}

	static NSString classNameDecodedForArchiveClassName (NSString inArchiveName)
	{
		return invokeObjcSelfClass!(NSString, "classNameDecodedForArchiveClassName:", NSString)(inArchiveName);
	}

	NSString classNameDecodedForArchiveClassName (NSString inArchiveName)
	{
		return invokeObjcSelf!(NSString, "classNameDecodedForArchiveClassName:", NSString)(inArchiveName);
	}

	void replaceObject (Object object, Object newObject)
	{
		return invokeObjcSelf!(void, "replaceObject:withObject:", Object, Object)(object, newObject);
	}
}

template TNSArchiverCallback ()
{
	Class classForArchiver ();
	Object replacementObjectForArchiver (NSArchiver archiver);
}

