/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSError;

import dstep.foundation.NSArray;
import dstep.foundation.NSDictionary;
import dstep.foundation.NSObject;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc : id;

import bindings = dstep.foundation.NSError_bindings;

const NSString NSCocoaErrorDomain;
const NSString NSPOSIXErrorDomain;
const NSString NSOSStatusErrorDomain;
const NSString NSMachErrorDomain;
const NSString NSUnderlyingErrorKey;
const NSString NSLocalizedDescriptionKey;
const NSString NSLocalizedFailureReasonErrorKey;
const NSString NSLocalizedRecoverySuggestionErrorKey;
const NSString NSLocalizedRecoveryOptionsErrorKey;
const NSString NSRecoveryAttempterErrorKey;
const NSString NSStringEncodingErrorKey;
const NSString NSURLErrorKey;
const NSString NSFilePathErrorKey;

static this ()
{
	NSCocoaErrorDomain = new NSString(bindings.NSCocoaErrorDomain);
	NSPOSIXErrorDomain = new NSString(bindings.NSPOSIXErrorDomain);
	NSOSStatusErrorDomain = new NSString(bindings.NSOSStatusErrorDomain);
	NSMachErrorDomain = new NSString(bindings.NSMachErrorDomain);
	NSUnderlyingErrorKey = new NSString(bindings.NSUnderlyingErrorKey);
	NSLocalizedDescriptionKey = new NSString(bindings.NSLocalizedDescriptionKey);
	NSLocalizedFailureReasonErrorKey = new NSString(bindings.NSLocalizedFailureReasonErrorKey);
	NSLocalizedRecoverySuggestionErrorKey = new NSString(bindings.NSLocalizedRecoverySuggestionErrorKey);
	NSLocalizedRecoveryOptionsErrorKey = new NSString(bindings.NSLocalizedRecoveryOptionsErrorKey);
	NSRecoveryAttempterErrorKey = new NSString(bindings.NSRecoveryAttempterErrorKey);
	NSStringEncodingErrorKey = new NSString(bindings.NSStringEncodingErrorKey);
	NSURLErrorKey = new NSString(bindings.NSURLErrorKey);
	NSFilePathErrorKey = new NSString(bindings.NSFilePathErrorKey);
}

class NSError : NSObject, INSCopying, INSCoding
{
	mixin ObjcWrap;

	Object initWithDomain (NSString domain, NSInteger code, NSDictionary dict)
	{
		return invokeObjcSelf!(Object, "initWithDomain:code:userInfo:", NSString, NSInteger, NSDictionary)(domain, code, dict);
	}

	this (NSString domain, NSInteger code, NSDictionary dict)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithDomain:code:userInfo:", NSString, NSInteger, NSDictionary)(objcObject, domain, code, dict);

		if (result)
			objcObject = ret;

		dObject = this;
	}

	static Object errorWithDomain (NSString domain, NSInteger code, NSDictionary dict)
	{
		return invokeObjcSelfClass!(Object, "errorWithDomain:code:userInfo:", NSString, NSInteger, NSDictionary)(domain, code, dict);
	}

	NSString domain ()
	{
		return invokeObjcSelf!(NSString, "domain");
	}

	NSInteger code ()
	{
		return invokeObjcSelf!(NSInteger, "code");
	}

	NSDictionary userInfo ()
	{
		return invokeObjcSelf!(NSDictionary, "userInfo");
	}

	NSString localizedDescription ()
	{
		return invokeObjcSelf!(NSString, "localizedDescription");
	}

	NSString localizedFailureReason ()
	{
		return invokeObjcSelf!(NSString, "localizedFailureReason");
	}

	NSString localizedRecoverySuggestion ()
	{
		return invokeObjcSelf!(NSString, "localizedRecoverySuggestion");
	}

	NSArray localizedRecoveryOptions ()
	{
		return invokeObjcSelf!(NSArray, "localizedRecoveryOptions");
	}

	Object recoveryAttempter ()
	{
		return invokeObjcSelf!(Object, "recoveryAttempter");
	}

	Object copyWithZone (NSZone* zone)
	{
		return invokeObjcSelf!(Object, "copyWithZone:", NSZone*)(zone);
	}

	void encodeWithCoder (NSCoder aCoder)
	{
		return invokeObjcSelf!(void, "encodeWithCoder:", NSCoder)(aCoder);
	}

	Object initWithCoder (NSCoder aDecoder)
	{
		return invokeObjcSelf!(Object, "initWithCoder:", NSCoder)(aDecoder);
	}

	this (NSCoder aDecoder)
	{
		objcObject = Bridge.invokeObjcClassMethod!(id, "alloc")(objcClass);
		id result = Bridge.invokeObjcMethod!(id, "initWithCoder:", NSCoder)(objcObject, aDecoder);

		if (result)
			objcObject = ret;

		dObject = this;
	}
}

template TNSErrorRecoveryAttempting ()
{
	void attemptRecoveryFromError (NSError error, NSUInteger recoveryOptionIndex, Object delegate_, SEL didRecoverSelector, void* contextInfo);
	bool attemptRecoveryFromError (NSError error, NSUInteger recoveryOptionIndex);
}

